from distutils.core import setup
setup(
    name='tornadocnauth',
    version='1.3',
    packages=['tornadocnauth'],
    author='zhangyangyu',
    author_email='phenixgeorge@126.com',
    description='providing similar interfaces as tornado.auth for popular websites OAuth in China',
    )
